<?php

namespace Drupal\ai_interpolator;

use Drupal\ai_interpolator\Exceptions\AiInterpolatorRuleNotFoundException;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Template\TwigEnvironment;

/**
 * Run one rule.
 */
class AiInterpolatorRuleRunner {

  /**
   * The entity type manager.
   */
  protected EntityTypeManager $entityType;

  /**
   * The twig environment.
   */
  protected TwigEnvironment $twig;

  /**
   * The field rule manager.
   */
  protected AiFieldRules $fieldRules;

  /**
   * The module handler.
   */
  protected ModuleHandler $moduleHandler;

  /**
   * Constructs a new AiInterpolatorRuleRunner object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type definition.
   * @param \Drupal\Core\Template\TwigEnvironment $twig
   *   Twig environment.
   * @param \Drupal\ai_interpolator\AiFieldRules $fieldRules
   *   The field rule manager.
   * @param \Drupal\Core\Extension\ModuleHandler $moduleHandler
   *   The module handler.
   */
  public function __construct(EntityTypeManager $entityTypeManager, TwigEnvironment $twig, AiFieldRules $fieldRules, ModuleHandler $moduleHandler) {
    $this->entityType = $entityTypeManager;
    $this->twig = $twig;
    $this->fieldRules = $fieldRules;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * Generate response.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity being worked on.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $fieldDefinition
   *   The field definition interface.
   * @param array $interpolatorConfig
   *   The interpolator config.
   *
   * @return bool
   *   Success or not.
   */
  public function generateResponse(EntityInterface $entity, FieldDefinitionInterface $fieldDefinition, array $interpolatorConfig) {
    // Get rule.
    $rule = $this->fieldRules->findRule($interpolatorConfig['rule']);

    if (!$rule) {
      throw new AiInterpolatorRuleNotFoundException('The rule could not be found: ' . $fieldDefinition->getType());
    }

    // Generate the real prompt if needed.
    $prompt = "";
    if (!empty($interpolatorConfig['mode']) && $interpolatorConfig['mode'] == 'token' && $this->moduleHandler->moduleExists('token')) {
      $prompt = $this->renderTokenPrompt($interpolatorConfig['token'], $entity);
    }
    elseif ($rule->needsPrompt()) {
      // Get tokens.
      $tokens = $rule->generateTokens($entity, $fieldDefinition, $interpolatorConfig);
      // Run rule.
      $prompt = $this->renderPrompt($interpolatorConfig['prompt'], $tokens);
    }

    // Generate values.
    $values = $rule->generate($prompt, $entity, $fieldDefinition, $interpolatorConfig);

    foreach ($values as $key => $value) {
      // Remove values that does not fit.
      if (!$rule->verifyValue($entity, $value, $fieldDefinition)) {
        unset($values[$key]);
      }
    }

    // Save values.
    if ($values && is_array($values)) {
      $rule->storeValues($entity, $values, $fieldDefinition);
    }
    return $entity;
  }

  /**
   * Render a prompt.
   *
   * @var string $prompt
   *   The prompt.
   * @var array $tokens
   *   The placeholders.
   *
   * @return string
   *   The rendered twig.
   */
  protected function renderPrompt($prompt, array $tokens) {
    // Get variables.
    $template = $this->twig->createTemplate(htmlspecialchars_decode($prompt));
    return $template->render($tokens);
  }

  /**
   * Render a tokenized prompt.
   *
   * @var string $prompt
   *   The prompt.
   * @var \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   *
   * @return string
   *   The rendered prompt.
   */
  protected function renderTokenPrompt($prompt, ContentEntityInterface $entity) {
    // Get variables.
    return \Drupal::token()->replace($prompt, [
      $entity->getEntityTypeId() => $entity,
      'user' => \Drupal::currentUser(),
    ]);
  }

}
